@@ -19,6 +19,7 @@ use std::ffi::OsStr;
 use std::fmt::{self, Debug};
 use std::fs::{self, File};
 use std::io::{Read, Write};
+use std::iter;
 use std::marker::PhantomData;
 use std::path::{Path, PathBuf};
 use std::process::{Command, Stdio};
@@ -140,9 +141,7 @@ pub enum MergeResult<'a> {
     Conflict(Vec<Conflict>),
     /// The merge is ready to be committed.
     ///
-    /// The command may be executed in order to create the commit from the merged tree. It is
-    /// expected that the relevant `GIT_` environment variables are used on the command to set the
-    /// author and committer information.
+    /// The command may be executed in order to create the commit from the merged tree.
     Ready(MergeCommand),
 
     #[doc(hidden)]
@@ -188,14 +187,14 @@ trait WorkareaGitContext {
 }
 
 /// Checkout a set of paths into a workarea.
-fn checkout<P>(ctx: &WorkareaGitContext, paths: &[P]) -> Result<()>
-    where P: AsRef<OsStr>,
+fn checkout<I, P>(ctx: &WorkareaGitContext, paths: I) -> Result<()>
+    where I: IntoIterator<Item = P>,
+          P: AsRef<OsStr>,
 {
-    // Checkout .gitmodules so that submodules work.
     let ls_files = ctx.cmd()
         .arg("ls-files")
         .arg("--")
-        .args(paths)
+        .args(paths.into_iter())
         .output()
         .chain_err(|| "failed to construct ls-files command")?;
     if !ls_files.status.success() {
@@ -325,7 +324,7 @@ impl PreparingGitWorkArea {
         // Explicitly do not check the return code; it is a failure.
 
         // Checkout .gitmodules so that submodules work.
-        checkout(self, &[".gitmodules"])
+        checkout(self, iter::once(".gitmodules"))
     }
 
     /// Run a git command in the workarea.
@@ -380,10 +379,7 @@ impl PreparingGitWorkArea {
 
         let gitmoduledir = self.context.gitdir().join("modules");
         Ok(submodule_config.into_iter()
-            .filter(|&(_, ref config)| {
-                config.get("path")
-                    .map_or(false, |path| gitmoduledir.join(path).exists())
-            })
+            .filter(|&(ref name, _)| gitmoduledir.join(name).exists())
             .collect())
     }
 
@@ -622,8 +618,9 @@ impl GitWorkArea {
     /// and fixes up Git's knowledge that they are there.
     ///
     /// All paths supported by Git's globbing and searching mechanisms are supported.
-    pub fn checkout<P>(&self, paths: &[P]) -> Result<()>
-        where P: AsRef<OsStr>,
+    pub fn checkout<I, P>(&self, paths: I) -> Result<()>
+        where I: IntoIterator<Item = P>,
+              P: AsRef<OsStr>,
     {
         checkout(self, paths)
     }
@@ -635,11 +632,6 @@ impl GitWorkArea {
     /// object. That commit object should then be stored in a reference using `git update-ref`.
     pub fn setup_merge<'a>(&'a self, bases: &[CommitId], base: &CommitId, topic: &CommitId)
                            -> Result<MergeResult<'a>> {
-        debug!(target: "git.workarea",
-               "merging {} into {}",
-               topic,
-               base);
-
         let merge_recursive = self.git()
             .arg("merge-recursive")
             .args(&bases.iter()
@@ -654,6 +646,29 @@ impl GitWorkArea {
             return Ok(MergeResult::Conflict(self.conflict_information()?));
         }
 
+        self.setup_merge_impl(base, topic)
+    }
+
+    /// Prepare a command to create a merge commit.
+    ///
+    /// The merge is performed, but only as a tree object. In order to create the actual commit
+    /// object, a successful merge returns a command which should be executed to create the commit
+    /// object. That commit object should then be stored in a reference using `git update-ref`.
+    pub fn setup_update_merge<'a>(&'a self, base: &CommitId, topic: &CommitId)
+                                  -> Result<MergeResult<'a>> {
+        self.setup_merge_impl(base, topic)
+    }
+
+    /// Prepare a command to create a merge commit.
+    ///
+    /// This supports choosing the merge strategy.
+    fn setup_merge_impl<'a>(&'a self, base: &CommitId, topic: &CommitId)
+                            -> Result<MergeResult<'a>> {
+        debug!(target: "git.workarea",
+               "merging {} into {}",
+               topic,
+               base);
+
         let write_tree = self.git()
             .arg("write-tree")
             .output()
